@@ -23,8 +23,6 @@
 
 #include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
 
-#include <gtkmm.h>
-
 #include <libglom/document/view.h>
 #include <libglom/connectionpool.h>
 #include <libglom/appstate.h>
@@ -35,6 +33,7 @@
 #include <libglom/calcinprogress.h>
 #include <libglom/document/bakery/view/view.h>
 #include <glom/bakery/busy_cursor.h>
+#include <gtkmm/treemodel.h>
 
 #include <libgdamm/set.h>
 #include <libgdamm/sqlbuilder.h>
@@ -70,8 +69,6 @@ public:
   virtual void set_document(Document* pDocument); //View override
   virtual void load_from_document(); //View override
 
-  static int count_rows_returned_by(const Glib::RefPtr<const Gnome::Gda::SqlBuilder>& sql_query);
-
   sharedptr<Field> change_column(const Glib::ustring& table_name, const sharedptr<const Field>& field_old, const sharedptr<const Field>& field, Gtk::Window* parent_window) const;
 
   typedef std::vector< sharedptr<Field> > type_vec_fields;
@@ -303,14 +300,6 @@ protected:
   type_vecConstLayoutFields get_table_fields_to_show_for_sequence(const Glib::ustring& table_name, const Document::type_list_layout_groups& mapGroupSequence) const;
   void get_table_fields_to_show_for_sequence_add_group(const Glib::ustring& table_name, const Privileges& table_privs, const type_vec_fields& all_db_fields, const sharedptr<LayoutGroup>& group, type_vecConstLayoutFields& vecFields) const;
 
-  /** Get the relationship (from the related table) into which the row button should navigate,
-   * or none if it should use the portal's directly related table itself.
-   * (If that should be chosen automatically, by looking at the fields in the portal.)
-   */
-  sharedptr<const UsesRelationship> get_portal_navigation_relationship_automatic(const sharedptr<LayoutItem_Portal>& portal) const;
-  sharedptr<const LayoutItem_Field> get_field_is_from_non_hidden_related_record(const sharedptr<const LayoutItem_Portal>& portal) const;
-  sharedptr<const LayoutItem_Field> get_field_identifies_non_hidden_related_record(const sharedptr<const LayoutItem_Portal>& portal, sharedptr<const Relationship>& used_in_relationship) const;
-
   bool get_primary_key_is_in_foundset(const FoundSet& found_set, const Gnome::Gda::Value& primary_key_value);
 
 
@@ -330,18 +319,6 @@ protected:
   typedef std::vector<Glib::ustring> type_vec_strings;
   static type_vec_strings util_vecStrings_from_Fields(const type_vec_fields& fields);
 
-  /** Add a @a user to the database, with the specified @a password, in the specified @a group.
-   * @result true if the addition succeeded.
-   */
-  bool add_user(const Glib::ustring& user, const Glib::ustring& password, const Glib::ustring& group);
-
-  /** Remove the @a user from the database.
-   * @result true if the removal succeeded.
-   */
-  bool remove_user(const Glib::ustring& user);
-
-  bool remove_user_from_group(const Glib::ustring& user, const Glib::ustring& group);
-
   bool set_database_owner_user(const Glib::ustring& user);
 
   /** Revoke any login rights from the user and remove it from any groups.
