@@ -18,17 +18,19 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <gtkmm.h>
 #include <glom/utility_widgets/cellrendererlist.h>
 #include <glom/mode_data/datawidget/cellrenderer_dblist.h>
 #include <glom/mode_data/datawidget/cellrenderer_buttonimage.h>
 #include <glom/mode_data/datawidget/cellrenderer_buttontext.h>
 #include <glom/utils_ui.h>
+#include <gtkmm/cellrenderertext.h>
+#include <gtkmm/stock.h>
 #include <libglom/data_structure/layout/layoutitem_field.h>
 #include <libglom/data_structure/layout/layoutitem_image.h>
 #include <libglom/utils.h>
 #include <libglom/db_utils.h>
 #include <libglom/data_structure/glomconversions.h>
+#include <iostream>
 
 
 namespace Glom
@@ -71,8 +73,14 @@ Gtk::CellRenderer* create_cell(const sharedptr<const LayoutItem>& layout_item, c
   sharedptr<const LayoutItem_Field> item_field = sharedptr<const LayoutItem_Field>::cast_dynamic(layout_item);
   if(item_field)
   {
+    //Ignore hiddent fields.
+    //For instance, these are generally added to DbTreeModels when they would not otherwise contain the primary key,
+    //so that the record can still be uniquely identified.
     if(item_field->get_hidden())
+    {
+      //std::cerr << G_STRFUNC << ": Returning 0 because the layout field is hidden. table_name=" << table_name << ", field name=" << item_field->get_name() << std::endl;
       return 0;
+    }
 
     switch(item_field->get_glom_type())
     {
@@ -90,14 +98,22 @@ Gtk::CellRenderer* create_cell(const sharedptr<const LayoutItem>& layout_item, c
       }
       default:
       {
-        if(item_field->get_formatting_used().get_has_choices())
+        const FieldFormatting& formatting = item_field->get_formatting_used();
+        if(formatting.get_has_choices())
         {
           CellRendererDbList* rendererList = Gtk::manage( new CellRendererDbList() );
           sharedptr<LayoutItem> unconst = sharedptr<LayoutItem>::cast_const(layout_item); //TODO: Avoid this.
           rendererList->set_layout_item(unconst, table_name);
           bool as_radio_buttons = false; //Can't really be done in a list, so we ignore it.
           rendererList->set_restrict_values_to_list(
-            item_field->get_formatting_used().get_choices_restricted(as_radio_buttons));
+            formatting.get_choices_restricted(as_radio_buttons));
+
+          //Set the choices.
+          //For related choices, that gets set when the value of a dependent field is set.
+          if(formatting.get_has_custom_choices())
+          {
+            rendererList->set_choices_fixed( formatting.get_choices_custom() );
+          }
 
           cell = rendererList;
         }
@@ -151,7 +167,10 @@ Gtk::CellRenderer* create_cell(const sharedptr<const LayoutItem>& layout_item, c
   }
 
   if(!cell)
+  {
+    std::cerr << G_STRFUNC << ": Returning 0 because no cell was created." << std::endl;
     return 0;
+  }
 
   //Use formatting:
   sharedptr<const LayoutItem_WithFormatting> item_withformatting =
