@@ -18,21 +18,19 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef GLOM_UTILITY_WIDGETS_DRAGBAR_H
-#define GLOM_UTILITY_WIDGETS_DRAGBAR_H
+#ifndef GLOM_PRINT_LAYOUT_TOOLBAR_H
+#define GLOM_PRINT_LAYOUT_TOOLBAR_H
 
 #include <gtkmm/box.h>
 #include <glom/mode_design/print_layouts/print_layout_toolbar_button.h>
 #include <gtkmm/window.h>
-#include <gtkmm/handlebox.h>
+#include <gtkmm/toolpalette.h>
 #include <gtkmm/toolitemgroup.h>
 
-#include <glom/utility_widgets/sidebar.h>
-
 namespace Glom
 {
 
-class PrintLayoutToolbar : public SideBar
+class PrintLayoutToolbar : public Gtk::ToolPalette
 {
 public:
   PrintLayoutToolbar();
@@ -46,4 +44,4 @@ private:
 
 } //namespace Glom
 
-#endif // GLOM_UTILITY_WIDGETS_DRAGBAR_H
+#endif // GLOM_PRINT_LAYOUT_TOOLBAR_H
