@@ -50,6 +50,10 @@ public:
 
   void set_page_setup(const Glib::RefPtr<Gtk::PageSetup>& page_setup);
   Glib::RefPtr<Gtk::PageSetup> get_page_setup();
+  Glib::RefPtr<const Gtk::PageSetup> get_page_setup() const;
+  
+  void set_page_count(guint count);
+  guint get_page_count() const;
 
   void set_zoom_percent(guint percent);
  
@@ -62,7 +66,13 @@ public:
 
   void remove_canvas_layout_item(const Glib::RefPtr<CanvasLayoutItem>& item);
 
-  void fill_with_data(const FoundSet& found_set);
+  /** If the item is a field from the System Preferences table,
+   * show the content instead of the field name,
+   * because it will be the same for all records.
+   */
+  void fill_with_data_system_preferences(const Glib::RefPtr<CanvasLayoutItem>& canvas_item, Document* document);
+
+  void fill_with_data(const FoundSet& found_set, bool avoid_page_margins);
 
   virtual void set_grid_gap(double gap = 20.0);
 
@@ -77,6 +87,16 @@ public:
    */
   void select_all(bool selected = true);
   
+  
+  Goocanvas::Bounds get_page_bounds(guint page_num) const;
+
+  /** Look for any items that overlap the @a canvas_item and move them down so that the no longer overlap.
+   * @param y_start Ignore any items whose y position is less than this.
+   * @param offset Move items down by this amount
+   * @param result The highest item that should be moved down to the start of the next page, if any:
+   */
+  Glib::RefPtr<CanvasLayoutItem> move_items_down(double y_start, double offset);
+  
 private:
 
 #ifndef GLOM_ENABLE_CLIENT_ONLY
@@ -88,14 +108,14 @@ private:
   void fill_layout_group(const sharedptr<LayoutGroup>& group);
 
   //These are not static, because they need access to the document:
-  void fill_with_data(const Glib::RefPtr<Goocanvas::Group>& canvas_group, const FoundSet& found_set);
+  void fill_with_data(const Glib::RefPtr<Goocanvas::Group>& canvas_group, const FoundSet& found_set, bool avoid_page_margins);
   void fill_with_data_portal(const Glib::RefPtr<CanvasLayoutItem>& canvas_item, const Gnome::Gda::Value& foreign_key_value);
-  static void set_canvas_item_field_value(const Glib::RefPtr<Goocanvas::Item> canvas_item, const sharedptr<LayoutItem_Field> field, const Gnome::Gda::Value& value);
-
-  static Glib::RefPtr<Goocanvas::Item> get_canvas_table_cell_child(const Glib::RefPtr<Goocanvas::Table>& table, int row, int col); //TODO: Add this to Goocanvas::Table.
+  static void set_canvas_item_field_value(const Glib::RefPtr<Goocanvas::Item>& canvas_item, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value);
   
   type_vecConstLayoutFields get_portal_fields_to_show(const sharedptr<LayoutItem_Portal>& portal);
 
+  void create_canvas_layout_item_and_add(const sharedptr<LayoutItem>& layout_item);
+
 #ifndef GLOM_ENABLE_CLIENT_ONLY
   sharedptr<LayoutItem_Portal> offer_related_records(const sharedptr<LayoutItem_Portal>& portal, Gtk::Window* parent);
   sharedptr<LayoutItem_Line> offer_line(const sharedptr<LayoutItem_Line>& portal, Gtk::Window* parent);
@@ -110,8 +130,13 @@ private:
   void on_dialog_format_hide();
 #endif
 
+  void update_page_bounds();
+  
   Glib::RefPtr<Goocanvas::Polyline> create_margin_line(double x1, double y1, double x2, double y2);
 
+  double get_page_height() const;
+  double get_page_height(double& margin_top, double& margin_bottom) const;
+
   Glib::ustring m_table_name;
   bool m_modified; //TODO: Actually check this?
 
@@ -121,7 +146,11 @@ private:
   //A rectangle to show the bounds:
   Glib::RefPtr<Goocanvas::Group> m_bounds_group; //the page and its margins.
   Glib::RefPtr<Goocanvas::Rect> m_bounds_rect;
-  Glib::RefPtr<Goocanvas::Polyline> m_margin_left, m_margin_right, m_margin_top, m_margin_bottom;
+  Glib::RefPtr<Goocanvas::Polyline> m_margin_left, m_margin_right;
+  
+  typedef std::vector< Glib::RefPtr<Goocanvas::Polyline> > type_vec_margins;
+  type_vec_margins m_vec_margin_tops;
+  type_vec_margins m_vec_margin_bottoms;
 
   //Context menu for existing items:
   Gtk::Menu* m_context_menu;
@@ -136,6 +165,7 @@ private:
   Dialog_TextFormatting* m_dialog_format;
 
   bool m_outline_visibility;
+  guint m_page_count;
 };
 
 } //namespace Glom
